@@ -152,6 +152,8 @@ Don't change without re-evaluating `cg-kw-re' (or all of cg.el).")
 
 (defconst cg-kw-rule-list
   '("SUBSTITUTE"
+    "PROTECT"
+    "UNPROTECT"
     "IFF"
     "ADDCOHORT" "REMCOHORT"
     "COPY"
@@ -376,7 +378,7 @@ CG-mode provides the following specific keyboard key bindings:
          nil				; KEYWORDS-ONLY
          'case-fold ; some keywords (e.g. x vs X) are case-sensitive,
                                         ; but that doesn't matter for highlighting
-         ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w"))
+         ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w") (?& . "w"))
          nil ;	  beginning-of-line		; SYNTAX-BEGIN
          (font-lock-syntactic-keywords . cg-font-lock-syntactic-keywords)
          (font-lock-syntactic-face-function . cg-font-lock-syntactic-face-function)))
@@ -398,8 +400,12 @@ CG-mode provides the following specific keyboard key bindings:
     ;; TODO: emacs 25 prefers `font-lock-ensure' and `font-lock-flush' over fontify
     (font-lock-fontify-buffer))
   (add-hook 'after-change-functions #'cg-after-change nil 'buffer-local)
-  (let ((buf (current-buffer)))
-    (run-with-idle-timer 1 'repeat 'cg-output-hl buf)))
+  (let* ((buf (current-buffer))
+         (hl-timer (run-with-idle-timer 1 'repeat 'cg-output-hl buf)))
+    (add-hook 'kill-buffer-hook
+              (lambda () (cancel-timer hl-timer))
+              nil
+              'local)))
 
 
 (defconst cg-font-lock-syntactic-keywords
