@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -35,9 +35,11 @@ public:
 
 	void runGrammarOnText(std::istream& input, std::ostream& output);
 
-	bool wordform_case;
-	bool print_word_forms;
-	bool print_only_first;
+	bool wordform_case = false;
+	bool print_word_forms = true;
+	bool print_only_first = false;
+	bool delimit_lexical_units = true; // Should cohorts be surrounded by ^$ ?
+	bool surface_readings = false;	   // Should readings have escaped symbols as if they were surface forms?
 
 	void testPR(std::ostream& output);
 
@@ -48,6 +50,16 @@ protected:
 	void mergeMappings(Cohort& cohort);
 
 private:
+	/**
+	 * Parse a stream variable from cleaned blank.
+	 *
+	 * Tries not to do anything more than what happens in GrammarApplicator_runGrammar.cpp, for
+	 * easy merging.
+	 *
+	 * @param cleaned something like "<STREAMCMD:SETVAR:forskjell_skilnad" (note: no trailing >).
+	 */
+	void parseStreamVar(const SingleWindow* cSWindow, UString& cleaned,
+			    uint32FlatHashMap& variables_set, uint32FlatHashSet& variables_rem, uint32SortedVector& variables_output);
 	void printReading(Reading* reading, std::ostream& output, ApertiumCasing casing, int firstlower);
 	void processReading(Reading* cReading, UChar* reading_string, Tag* wform);
 	void processReading(Reading* cReading, UString& reading_string, Tag* wform);
