@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -100,6 +100,9 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	if (grammar->text_delimiters) {
 		fields |= BINF_TEXT_DELIMS;
 	}
+	if (grammar->addcohort_attach) {
+		fields |= BINF_ADDCOHORT_ATTACH;
+	}
 
 	u32tmp = hton32(fields);
 	fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
@@ -107,16 +110,17 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	if (grammar->mapping_prefix) {
 		ucnv_reset(conv);
 		i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, &grammar->mapping_prefix, 1, &err);
-		u32tmp = hton32((uint32_t)i32tmp);
+		u32tmp = hton32(UI32(i32tmp));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 		fwrite_throw(&cbuffers[0][0], i32tmp, 1, output);
 	}
 
-	if (!grammar->single_tags_list.empty()) {
-		u32tmp = hton32((uint32_t)grammar->single_tags_list.size());
+	if (grammar->num_tags) {
+		u32tmp = hton32(UI32(grammar->num_tags));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
-	for (auto t : grammar->single_tags_list) {
+	for (size_t i = 0; i < grammar->num_tags; ++i) {
+		auto t = grammar->single_tags_list[i];
 		uint32_t fields = 0;
 		buffer.str("");
 		buffer.clear();
@@ -159,7 +163,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 		if (!t->tag.empty()) {
 			fields |= (1 << 8);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, t->tag.c_str(), static_cast<int32_t>(t->tag.size()), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, t->tag.c_str(), SI32(t->tag.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
@@ -176,22 +180,26 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 
 		if (t->vs_sets) {
 			fields |= (1 << 10);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(t->vs_sets->size()));
+			writeSwapped<uint32_t>(buffer, UI32(t->vs_sets->size()));
 			for (auto iter : *t->vs_sets) {
 				writeSwapped(buffer, iter->number);
 			}
 		}
 		if (t->vs_names) {
 			fields |= (1 << 11);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(t->vs_names->size()));
-			for (auto iter : *t->vs_names) {
+			writeSwapped<uint32_t>(buffer, UI32(t->vs_names->size()));
+			for (const auto& iter : *t->vs_names) {
 				ucnv_reset(conv);
-				i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, iter.c_str(), static_cast<int32_t>(iter.size()), &err);
+				i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, iter.c_str(), SI32(iter.size()), &err);
 				writeSwapped(buffer, i32tmp);
 				buffer.write(&cbuffers[0][0], i32tmp);
 			}
 		}
 		// 1 << 12 used above
+		if ((t->type & T_VARIABLE) && t->variable_hash) {
+			fields |= (1 << 13);
+			writeSwapped(buffer, t->variable_hash);
+		}
 
 		u32tmp = hton32(fields);
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
@@ -200,7 +208,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->reopen_mappings.empty()) {
-		u32tmp = hton32((uint32_t)grammar->reopen_mappings.size());
+		u32tmp = hton32(UI32(grammar->reopen_mappings.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
 	for (auto iter : grammar->reopen_mappings) {
@@ -209,7 +217,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->preferred_targets.empty()) {
-		u32tmp = hton32((uint32_t)grammar->preferred_targets.size());
+		u32tmp = hton32(UI32(grammar->preferred_targets.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
 	for (auto iter : grammar->preferred_targets) {
@@ -218,10 +226,10 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->parentheses.empty()) {
-		u32tmp = hton32((uint32_t)grammar->parentheses.size());
+		u32tmp = hton32(UI32(grammar->parentheses.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
-	for (auto iter_par : grammar->parentheses) {
+	for (const auto& iter_par : grammar->parentheses) {
 		u32tmp = hton32(iter_par.first);
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 		u32tmp = hton32(iter_par.second);
@@ -229,10 +237,10 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->anchors.empty()) {
-		u32tmp = hton32((uint32_t)grammar->anchors.size());
+		u32tmp = hton32(UI32(grammar->anchors.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
-	for (auto iter_anchor : grammar->anchors) {
+	for (const auto& iter_anchor : grammar->anchors) {
 		u32tmp = hton32(iter_anchor.first);
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 		u32tmp = hton32(iter_anchor.second);
@@ -240,7 +248,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->sets_list.empty()) {
-		u32tmp = hton32((uint32_t)grammar->sets_list.size());
+		u32tmp = hton32(UI32(grammar->sets_list.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
 	for (auto s : grammar->sets_list) {
@@ -259,21 +267,21 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 		}
 		if (!s->getNonEmpty().empty()) {
 			fields |= (1 << 3);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->trie.size()));
+			writeSwapped<uint32_t>(buffer, UI32(s->trie.size()));
 			trie_serialize(s->trie, buffer);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->trie_special.size()));
+			writeSwapped<uint32_t>(buffer, UI32(s->trie_special.size()));
 			trie_serialize(s->trie_special, buffer);
 		}
 		if (!s->set_ops.empty()) {
 			fields |= (1 << 4);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->set_ops.size()));
+			writeSwapped<uint32_t>(buffer, UI32(s->set_ops.size()));
 			for (auto iter : s->set_ops) {
 				writeSwapped(buffer, iter);
 			}
 		}
 		if (!s->sets.empty()) {
 			fields |= (1 << 5);
-			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->sets.size()));
+			writeSwapped<uint32_t>(buffer, UI32(s->sets.size()));
 			for (auto iter : s->sets) {
 				writeSwapped(buffer, iter);
 			}
@@ -281,7 +289,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 		if (s->type & ST_STATIC) {
 			fields |= (1 << 6);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, s->name.c_str(), static_cast<int32_t>(s->name.size()), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, s->name.c_str(), SI32(s->name.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
@@ -309,7 +317,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 
 	seen_uint32.clear();
 	if (!grammar->contexts.empty()) {
-		u32tmp = hton32((uint32_t)grammar->contexts.size());
+		u32tmp = hton32(UI32(grammar->contexts.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
 	for (auto& cntx : grammar->contexts) {
@@ -317,7 +325,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	}
 
 	if (!grammar->rule_by_number.empty()) {
-		u32tmp = hton32((uint32_t)grammar->rule_by_number.size());
+		u32tmp = hton32(UI32(grammar->rule_by_number.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 	}
 	for (auto r : grammar->rule_by_number) {
@@ -344,7 +352,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 		if (!r->name.empty()) {
 			fields |= (1 << 4);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, r->name.c_str(), static_cast<int32_t>(r->name.size()), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, r->name.c_str(), SI32(r->name.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
@@ -454,9 +462,9 @@ void BinaryGrammar::writeContextualTest(ContextualTest* t, FILE* output) {
 	}
 	if (t->pos) {
 		fields |= (1 << 1);
-		writeSwapped(buffer, static_cast<uint32_t>(t->pos & 0xFFFFFFFF));
+		writeSwapped(buffer, UI32(t->pos & 0xFFFFFFFF));
 		if (t->pos & POS_64BIT) {
-			writeSwapped(buffer, static_cast<uint32_t>((t->pos >> 32) & 0xFFFFFFFF));
+			writeSwapped(buffer, UI32((t->pos >> 32) & 0xFFFFFFFF));
 		}
 	}
 	if (t->offset) {
@@ -504,7 +512,7 @@ void BinaryGrammar::writeContextualTest(ContextualTest* t, FILE* output) {
 	fwrite_throw(str.c_str(), str.size(), 1, output);
 
 	if (!t->ors.empty()) {
-		u32tmp = hton32((uint32_t)t->ors.size());
+		u32tmp = hton32(UI32(t->ors.size()));
 		fwrite_throw(&u32tmp, sizeof(u32tmp), 1, output);
 
 		for (auto iter : t->ors) {
