@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -32,9 +32,9 @@ public:
 	FSTApplicator(std::ostream& ux_err);
 	void runGrammarOnText(std::istream& input, std::ostream& output);
 
-	double wfactor;
-	UString wtag;
-	UString sub_delims;
+	double wfactor = 1.0;
+	UString wtag{'W'};
+	UString sub_delims{'#'};
 };
 }
 
