@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -25,25 +25,6 @@
 
 namespace CG3 {
 
-Grammar::Grammar()
-  : ux_stderr(nullptr)
-  , ux_stdout(nullptr)
-  , has_dep(false)
-  , has_bag_of_tags(false)
-  , has_relations(false)
-  , has_encl_final(false)
-  , has_protect(false)
-  , is_binary(false)
-  , sub_readings_ltr(false)
-  , grammar_size(0)
-  , mapping_prefix('@')
-  , lines(0)
-  , verbosity_level(0)
-  , total_time(0)
-{
-	// Nothing in the actual body...
-}
-
 Grammar::~Grammar() {
 	for (auto iter_set : sets_list) {
 		destroySet(iter_set);
@@ -61,7 +42,7 @@ Grammar::~Grammar() {
 		delete iter_rules;
 	}
 
-	for (auto cntx : contexts) {
+	for (auto& cntx : contexts) {
 		delete cntx.second;
 	}
 }
@@ -248,16 +229,16 @@ void Grammar::addSet(Set*& to) {
 }
 
 Set* Grammar::getSet(uint32_t which) const {
-	Setuint32HashMap::const_iterator iter = sets_by_contents.find(which);
+	auto iter = sets_by_contents.find(which);
 	if (iter != sets_by_contents.end()) {
 		return iter->second;
 	}
 	else {
-		uint32FlatHashMap::const_iterator iter = sets_by_name.find(which);
+		auto iter = sets_by_name.find(which);
 		if (iter != sets_by_name.end()) {
-			Setuint32HashMap::const_iterator citer = sets_by_contents.find(iter->second);
+			auto citer = sets_by_contents.find(iter->second);
 			if (citer != sets_by_contents.end()) {
-				set_name_seeds_t::const_iterator iter2 = set_name_seeds.find(citer->second->name);
+				auto iter2 = set_name_seeds.find(citer->second->name);
 				if (iter2 != set_name_seeds.end()) {
 					return getSet(iter->second + iter2->second);
 				}
@@ -290,7 +271,7 @@ void Grammar::addSetToList(Set* s) {
 				}
 			}
 			sets_list.push_back(s);
-			s->number = static_cast<uint32_t>(sets_list.size() - 1);
+			s->number = UI32(sets_list.size() - 1);
 		}
 	}
 }
@@ -444,7 +425,7 @@ Rule* Grammar::allocateRule() {
 }
 
 void Grammar::addRule(Rule* rule) {
-	rule->number = static_cast<uint32_t>(rule_by_number.size());
+	rule->number = UI32(rule_by_number.size());
 	rule_by_number.push_back(rule);
 }
 
@@ -481,7 +462,6 @@ Tag* Grammar::allocateTag(const UString& txt) {
 }
 
 Tag* Grammar::addTag(Tag* tag) {
-	tag->type |= T_GRAMMAR;
 	uint32_t hash = tag->rehash();
 	for (uint32_t seed = 0; seed < 10000; seed++) {
 		uint32_t ih = hash + seed;
@@ -505,7 +485,7 @@ Tag* Grammar::addTag(Tag* tag) {
 			tag->seed = seed;
 			hash = tag->rehash();
 			single_tags_list.push_back(tag);
-			tag->number = static_cast<uint32_t>(single_tags_list.size() - 1);
+			tag->number = UI32(single_tags_list.size() - 1);
 			single_tags[hash] = tag;
 			break;
 		}
@@ -549,7 +529,7 @@ ContextualTest* Grammar::addContextualTest(ContextualTest* t) {
 	}
 
 	for (uint32_t seed = 0; seed < 1000; ++seed) {
-		contexts_t::iterator cit = contexts.find(t->hash + seed);
+		auto cit = contexts.find(t->hash + seed);
 		if (cit == contexts.end()) {
 			contexts[t->hash + seed] = t;
 			t->hash += seed;
@@ -583,14 +563,14 @@ void Grammar::addTemplate(ContextualTest* test, const UChar* name) {
 
 void Grammar::addAnchor(const UChar* to, uint32_t at, bool primary) {
 	uint32_t ah = allocateTag(to)->hash;
-	uint32FlatHashMap::iterator it = anchors.find(ah);
+	auto it = anchors.find(ah);
 	if (primary && it != anchors.end()) {
 		u_fprintf(ux_stderr, "Error: Redefinition attempt for anchor '%S' on line %u!\n", to, lines);
 		CG3Quit(1);
 	}
 	if (at > rule_by_number.size()) {
 		u_fprintf(ux_stderr, "Warning: No corresponding rule available for anchor '%S' on line %u!\n", to, lines);
-		at = static_cast<uint32_t>(rule_by_number.size());
+		at = UI32(rule_by_number.size());
 	}
 	if (it == anchors.end()) {
 		anchors[ah] = at;
@@ -617,7 +597,7 @@ void Grammar::renameAllRules() {
 }
 
 void Grammar::reindex(bool unused_sets, bool used_tags) {
-	for (auto dset : sets_by_contents) {
+	for (const auto& dset : sets_by_contents) {
 		if (dset.second->number == std::numeric_limits<uint32_t>::max()) {
 			dset.second->type |= ST_USED;
 			continue;
@@ -628,7 +608,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		dset.second->number = 0;
 	}
 
-	for (auto sset : static_sets) {
+	for (const auto& sset : static_sets) {
 		uint32_t sh = hash_value(sset);
 		if (set_alias.find(sh) != set_alias.end()) {
 			u_fprintf(ux_stderr, "Error: Static set %S is an alias; only real sets may be made static!\n", sset.c_str());
@@ -687,7 +667,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 		for (auto iter : regex_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(iter, titer->tag.c_str(), static_cast<int32_t>(titer->tag.size()), &status);
+			uregex_setText(iter, titer->tag.c_str(), SI32(titer->tag.size()), &status);
 			if (status == U_ZERO_ERROR) {
 				if (uregex_find(iter, -1, &status)) {
 					titer->type |= T_TEXTUAL;
@@ -772,7 +752,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 
 	if (unused_sets) {
 		u_fprintf(ux_stdout, "Unused sets:\n");
-		for (auto rset : sets_by_contents) {
+		for (const auto& rset : sets_by_contents) {
 			if (!(rset.second->type & ST_USED) && !rset.second->name.empty() && maybe_used_sets.count(rset.second) == 0) {
 				if (!is_internal(rset.second->name)) {
 					u_fprintf(ux_stdout, "Line %u set %S\n", rset.second->line, rset.second->name.c_str());
@@ -785,13 +765,13 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 
 	// Stuff below this line is not optional...
 
-	for (auto tset : sets_by_contents) {
+	for (const auto& tset : sets_by_contents) {
 		if (tset.second->type & ST_USED) {
 			addSetToList(tset.second);
 		}
 	}
 
-	for (auto iter_tags : single_tags) {
+	for (const auto& iter_tags : single_tags) {
 		Tag* tag = iter_tags.second;
 		if (tag->tag[0] == mapping_prefix) {
 			tag->type |= T_MAPPING;
