@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -46,7 +46,7 @@ inline bool TagSet_SubsetOf_TSet(const TagSortedVector& a, const T& b) {
 		return false;
 	}
 	//*/
-	typename T::const_iterator bi = b.lower_bound((*a.begin())->hash);
+	auto bi = b.lower_bound((*a.begin())->hash);
 	for (auto ai : a) {
 		while (bi != b.end() && *bi < ai->hash) {
 			++bi;
@@ -65,20 +65,21 @@ inline bool TagSet_SubsetOf_TSet(const TagSortedVector& a, const T& b) {
 * @param[in] tag The tag to test against; only uses the hash and regexp members
 */
 uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bool bypass_index) {
+	UErrorCode status = U_ZERO_ERROR;
+	int32_t gc = uregex_groupCount(tag.regexp, &status);
 	uint32_t match = 0;
 	uint32_t ih = hash_value(tag.hash, test);
 	if (!bypass_index && index_matches(index_regexp_no, ih)) {
 		match = 0;
 	}
-	else if (!bypass_index && index_matches(index_regexp_yes, ih)) {
+	else if (!bypass_index && gc == 0 && index_matches(index_regexp_yes, ih)) {
 		match = test;
 	}
 	else {
-		const Tag& itag = *(single_tags.find(test)->second);
-		UErrorCode status = U_ZERO_ERROR;
-		uregex_setText(tag.regexp, itag.tag.c_str(), static_cast<int32_t>(itag.tag.size()), &status);
+		const Tag& itag = *(grammar->single_tags.find(test)->second);
+		uregex_setText(tag.regexp, itag.tag.c_str(), SI32(itag.tag.size()), &status);
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_setText(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 			CG3Quit(1);
 		}
 		status = U_ZERO_ERROR;
@@ -86,11 +87,10 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 			match = itag.hash;
 		}
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_find(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_find(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 			CG3Quit(1);
 		}
 		if (match) {
-			int32_t gc = uregex_groupCount(tag.regexp, &status);
 			if (gc > 0 && regexgrps.second != 0) {
 				UChar tmp[1024];
 				for (int i = 1; i <= gc; ++i) {
@@ -124,7 +124,7 @@ uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, boo
 		match = test;
 	}
 	else {
-		const Tag& itag = *(single_tags.find(test)->second);
+		const Tag& itag = *(grammar->single_tags.find(test)->second);
 		if (ux_strCaseCompare(tag.tag, itag.tag)) {
 			match = itag.hash;
 		}
@@ -140,19 +140,20 @@ uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, boo
 
 // ToDo: Remove for real ordered mode
 uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Tag& tag, bool bypass_index) {
+	UErrorCode status = U_ZERO_ERROR;
+	int32_t gc = uregex_groupCount(tag.regexp, &status);
 	uint32_t match = 0;
 	uint32_t ih = hash_value(reading.tags_string_hash, tag.hash);
 	if (!bypass_index && index_matches(index_regexp_no, ih)) {
 		match = 0;
 	}
-	else if (!bypass_index && index_matches(index_regexp_yes, ih)) {
+	else if (!bypass_index && gc == 0 && index_matches(index_regexp_yes, ih)) {
 		match = reading.tags_string_hash;
 	}
 	else {
-		UErrorCode status = U_ZERO_ERROR;
-		uregex_setText(tag.regexp, reading.tags_string.c_str(), static_cast<int32_t>(reading.tags_string.size()), &status);
+		uregex_setText(tag.regexp, reading.tags_string.c_str(), SI32(reading.tags_string.size()), &status);
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_setText(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 			CG3Quit(1);
 		}
 		status = U_ZERO_ERROR;
@@ -160,11 +161,10 @@ uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Ta
 			match = reading.tags_string_hash;
 		}
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_find(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_find(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 			CG3Quit(1);
 		}
 		if (match) {
-			int32_t gc = uregex_groupCount(tag.regexp, &status);
 			// ToDo: Allow regex captures from dependency target contexts without any captures in normal target contexts
 			if (gc > 0 && regexgrps.second != 0) {
 				UChar tmp[1024];
@@ -255,7 +255,7 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 	else if (tag.type & T_META) {
 		if (tag.regexp && !reading.parent->text.empty()) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(tag.regexp, reading.parent->text.c_str(), static_cast<int32_t>(reading.parent->text.size()), &status);
+			uregex_setText(tag.regexp, reading.parent->text.c_str(), SI32(reading.parent->text.size()), &status);
 			if (status != U_ZERO_ERROR) {
 				u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 				CG3Quit(1);
@@ -325,7 +325,7 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		}
 		else {
 			for (auto mter : reading.tags_textual) {
-				const Tag& itag = *(single_tags.find(mter)->second);
+				const Tag& itag = *(grammar->single_tags.find(mter)->second);
 				if (!(itag.type & (T_BASEFORM | T_WORDFORM))) {
 					match = itag.hash;
 					if (unif_mode) {
@@ -346,7 +346,7 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		}
 	}
 	else if (tag.type & T_NUMERICAL) {
-		for (auto mter : reading.tags_numerical) {
+		for (const auto& mter : reading.tags_numerical) {
 			const Tag& itag = *(mter.second);
 			double compval = tag.comparison_val;
 			if (compval <= NUMERIC_MIN) {
@@ -471,13 +471,28 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		}
 	}
 	else if (tag.type & T_VARIABLE) {
-		if (variables.find(tag.comparison_hash) == variables.end()) {
-			//u_fprintf(ux_stderr, "Info: %S failed.\n", tag.tag.c_str());
-			match = 0;
-		}
-		else {
-			//u_fprintf(ux_stderr, "Info: %S matched.\n", tag.tag.c_str());
-			match = tag.hash;
+		match = 0;
+		auto it = variables.find(tag.comparison_hash);
+		if (it != variables.end()) {
+			if (tag.variable_hash == 0) {
+				match = tag.hash;
+			}
+			else {
+				auto comp = grammar->single_tags.find(tag.variable_hash)->second;
+				if (comp->type & T_REGEXP) {
+					if (doesTagMatchRegexp(it->second, *comp, bypass_index)) {
+						match = tag.hash;
+					}
+				}
+				else if (comp->type & T_CASE_INSENSITIVE) {
+					if (doesTagMatchIcase(it->second, *comp, bypass_index)) {
+						match = tag.hash;
+					}
+				}
+				else if (comp->hash == it->second) {
+					match = tag.hash;
+				}
+			}
 		}
 	}
 	else if (tag.type & T_PAR_LEFT) {
@@ -572,8 +587,8 @@ bool GrammarApplicator::doesSetMatchReading_tags(const Reading& reading, const S
 	// If there are no special circumstances the first test boils down to finding whether the tag stores intersect
 	// 80% of calls try this first.
 	if (!theset.trie.empty() && !reading.tags_plain.empty()) {
-		trie_t::const_iterator iiter = theset.trie.lower_bound(single_tags.find(reading.tags_plain.front())->second);
-		uint32SortedVector::const_iterator oiter = reading.tags_plain.lower_bound(theset.trie.begin()->first->hash);
+		auto iiter = theset.trie.lower_bound(grammar->single_tags.find(reading.tags_plain.front())->second);
+		auto oiter = reading.tags_plain.lower_bound(theset.trie.begin()->first->hash);
 		while (oiter != reading.tags_plain.end() && iiter != theset.trie.end()) {
 			if (*oiter == iiter->first->hash) {
 				if (iiter->second.terminal) {
@@ -892,6 +907,8 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 		lists[3] = &cohort.ignored;
 	}
 
+	bool non_match_was_sub = false;
+
 	for (auto list : lists) {
 		if (list == nullptr) {
 			continue;
@@ -901,6 +918,7 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 				// ToDo: Barriers need some way to escape sub-readings
 				reading = get_sub_reading(reading, context->test->offset_sub);
 				if (!reading) {
+					non_match_was_sub = true;
 					continue;
 				}
 			}
@@ -922,7 +940,7 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 
 	if (context && !context->matched_target) {
 		if (!grammar->sets_any || set >= grammar->sets_any->size() || !grammar->sets_any->test(set)) {
-			if (set < cohort.possible_sets.size()) {
+			if (!non_match_was_sub && set < cohort.possible_sets.size()) {
 				cohort.possible_sets.reset(set);
 			}
 		}
