@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -34,7 +34,7 @@ inline bool testStringAgainst(const UString& str, std::vector<URegularExpression
 
 	for (size_t i = 0; i < rxs.size(); ++i) {
 		UErrorCode status = U_ZERO_ERROR;
-		uregex_setText(rxs[i], str.c_str(), static_cast<int32_t>(str.size()), &status);
+		uregex_setText(rxs[i], str.c_str(), SI32(str.size()), &status);
 		if (status != U_ZERO_ERROR) {
 			CG3Quit(1);
 		}
@@ -151,40 +151,8 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 
 	while (!input.eof()) {
 		++lines;
-		size_t offset = 0, packoff = 0;
-		// Read as much of the next line as will fit in the current buffer
-		while (u_fgets(&line[offset], static_cast<int32_t>(line.size() - offset - 1), input)) {
-			// Copy the segment just read to cleaned
-			for (size_t i = offset; i < line.size(); ++i) {
-				// Only copy one space character, regardless of how many are in input
-				if (ISSPACE(line[i]) && !ISNL(line[i])) {
-					cleaned[packoff++] = ' ';
-					while (ISSPACE(line[i]) && !ISNL(line[i])) {
-						++i;
-					}
-				}
-				// Break if there is a newline
-				if (ISNL(line[i])) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					goto gotaline; // Oh how I wish C++ had break 2;
-				}
-				if (line[i] == 0) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					break;
-				}
-				cleaned[packoff++] = line[i];
-			}
-			// Either buffer wasn't big enough, or someone fed us malformed data thinking U+0085 is ellipsis when it in fact is Next Line (NEL)
-			line = cleaned;
-			offset = packoff;
-			if (packoff > line.size() / 2) {
-				// If we reached this, buffer wasn't big enough. Double the size of the buffer and try again.
-				line.resize(line.size() * 2, 0);
-				cleaned.resize(line.size() + 1, 0);
-			}
-		}
+		auto packoff = get_line_clean(line, cleaned, input);
 
-	gotaline:
 		// Trim trailing whitespace
 		while (cleaned[0] && ISSPACE(cleaned[packoff - 1])) {
 			cleaned[packoff - 1] = 0;
@@ -488,6 +456,12 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 					if (verbosity_level > 0) {
 						u_fprintf(ux_stderr, "Info: FLUSH encountered on line %u. Flushing...\n", numLines);
 					}
+
+					auto backSWindow = gWindow->back();
+					if (backSWindow) {
+						backSWindow->flush_after = true;
+					}
+
 					if (cCohort && cSWindow) {
 						splitAllMappings(all_mappings, *cCohort, true);
 						cSWindow->appendCohort(cCohort);
@@ -519,7 +493,9 @@ void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& outp
 						free_swindow(tmp);
 						gWindow->previous.erase(gWindow->previous.begin());
 					}
-					u_fprintf(output, "%S", &line[0]);
+					if (!backSWindow) {
+						u_fprintf(output, "%S", &line[0]);
+					}
 					line[0] = 0;
 					variables.clear();
 					u_fflush(output);
