@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -75,7 +75,7 @@ bool GrammarApplicator::updateRuleToCohorts(Cohort& c, const uint32_t& rsit) {
 
 bool GrammarApplicator::updateValidRules(const uint32IntervalVector& rules, uint32IntervalVector& intersects, const uint32_t& hash, Reading& reading) {
 	size_t os = intersects.size();
-	Grammar::rules_by_tag_t::const_iterator it = grammar->rules_by_tag.find(hash);
+	auto it = grammar->rules_by_tag.find(hash);
 	if (it != grammar->rules_by_tag.end()) {
 		Cohort& c = *(reading.parent);
 		for (auto rsit : (it->second)) {
@@ -253,7 +253,7 @@ Reading* GrammarApplicator::get_sub_reading(Reading* tr, int sub_reading) {
 					}                                                               \
 					auto stag = doesTagMatchReading(reading, *tt, false, true);     \
 					if (stag) {                                                     \
-						(taglist)->insert(it, single_tags.find(stag)->second);      \
+						(taglist)->insert(it, grammar->single_tags.find(stag)->second); \
 					}                                                               \
 				}                                                                   \
 				continue;                                                           \
@@ -271,7 +271,7 @@ Reading* GrammarApplicator::get_sub_reading(Reading* tr, int sub_reading) {
 				}                                                               \
 				auto stag = doesTagMatchReading(reading, *tt, false, true);     \
 				if (stag) {                                                     \
-					tt = single_tags.find(stag)->second;                        \
+					tt = grammar->single_tags.find(stag)->second;               \
 				}                                                               \
 			}                                                                   \
 		}                                                                       \
@@ -394,7 +394,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 		if (debug_level > 1) {
 			std::cerr << "DEBUG: " << cohortset->size() << "/" << current.cohorts.size() << " = " << double(cohortset->size()) / double(current.cohorts.size()) << std::endl;
 		}
-		for (CohortSet::const_iterator rocit = cohortset->begin(); rocit != cohortset->end();) {
+		for (auto rocit = cohortset->cbegin(); rocit != cohortset->cend();) {
 			Cohort* cohort = *rocit;
 			++rocit;
 
@@ -516,8 +516,8 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 			regexgrps_c.reserve(std::max(regexgrps_c.size(), cohort->readings.size()));
 
 			size_t used_unif = 0;
-			unif_tags_store.resize(std::max(unif_tags_store.size(), cohort->readings.size()));
-			unif_sets_store.resize(std::max(unif_sets_store.size(), cohort->readings.size()));
+			unif_tags_store.resize(std::max(unif_tags_store.size(), cohort->readings.size() + 1));
+			unif_sets_store.resize(std::max(unif_sets_store.size(), cohort->readings.size() + 1));
 
 			// This loop figures out which readings, if any, that are valid targets for the current rule
 			// Criteria for valid is that the reading must match both target and all contextual tests
@@ -553,7 +553,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				// Check if any previous reading of this cohort had the same plain signature, and if so just copy their results
 				// This cache is cleared on a per-cohort basis
 				if (!(set.type & (ST_SPECIAL | ST_MAPPING | ST_CHILD_UNIFY)) && !readings_plain.empty()) {
-					readings_plain_t::const_iterator rpit = readings_plain.find(reading->hash_plain);
+					auto rpit = readings_plain.find(reading->hash_plain);
 					if (rpit != readings_plain.end()) {
 						reading->matched_target = rpit->second->matched_target;
 						reading->matched_tests = rpit->second->matched_tests;
@@ -616,7 +616,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 							// Keeps track of where we have been, to prevent infinite recursion in trees with loops
 							dep_deep_seen.clear();
 							// Reset the counters for which types of CohortIterator we have in play
-							std::fill(ci_depths.begin(), ci_depths.end(), static_cast<uint32_t>(0));
+							std::fill(ci_depths.begin(), ci_depths.end(), UI32(0));
 							tmpl_cntx.clear();
 							// Run the contextual test...
 							if (!(test->pos & POS_PASS_ORIGIN) && (no_pass_origin || (test->pos & POS_NO_PASS_ORIGIN))) {
@@ -662,10 +662,13 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						++rule.num_match;
 
 						if (regex_prop && i && !regexgrps_c.empty()) {
-							auto it = regexgrps_c.find(cohort->readings[i - 1]->number);
-							if (it != regexgrps_c.end()) {
-								regexgrps_c.insert(std::make_pair(reading->number, it->second));
-								regexgrps_z.insert(std::make_pair(reading->number, regexgrps_z.find(cohort->readings[i - 1]->number)->second));
+							for (auto z = i; z > 0; --z) {
+								auto it = regexgrps_c.find(cohort->readings[z - 1]->number);
+								if (it != regexgrps_c.end()) {
+									regexgrps_c.insert(std::make_pair(reading->number, it->second));
+									regexgrps_z.insert(std::make_pair(reading->number, regexgrps_z.find(cohort->readings[z - 1]->number)->second));
+									break;
+								}
 							}
 						}
 					}
@@ -769,12 +772,12 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								continue;
 							}
 							tags.reserve(tags.size() + nt.size() - 2);
-							tags[i] = single_tags[nt[2]];
+							tags[i] = grammar->single_tags[nt[2]];
 							for (size_t j = 3, k = 1; j < nt.size(); ++j) {
-								if (single_tags[nt[j]]->type & T_DEPENDENCY) {
+								if (grammar->single_tags[nt[j]]->type & T_DEPENDENCY) {
 									continue;
 								}
-								tags.insert(tags.begin() + i + k, single_tags[nt[j]]);
+								tags.insert(tags.begin() + i + k, grammar->single_tags[nt[j]]);
 								++k;
 							}
 						}
@@ -812,6 +815,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 				current.parent->cohort_map[cCohort->global_number] = cCohort;
 				current.parent->dep_window[cCohort->global_number] = cCohort;
+				if (grammar->addcohort_attach && (type == K_ADDCOHORT_BEFORE || type == K_ADDCOHORT_AFTER)) {
+					attachParentChild(*cohort, *cCohort);
+				}
 
 				if (cCohort->readings.empty()) {
 					initEmptyCohort(*cCohort);
@@ -859,7 +865,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				}
 
 				foreach (iter, current.cohorts) {
-					(*iter)->local_number = static_cast<uint32_t>(std::distance(current.cohorts.begin(), iter));
+					(*iter)->local_number = UI32(std::distance(current.cohorts.begin(), iter));
 				}
 				gWindow->rebuildCohortLinks();
 
@@ -898,13 +904,13 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					cohort->prev->removed.push_back(cohort);
 				}
 				cohort->detach();
-				for (auto cm : current.parent->cohort_map) {
+				for (auto& cm : current.parent->cohort_map) {
 					cm.second->dep_children.erase(cohort->dep_self);
 				}
 				current.parent->cohort_map.erase(cohort->global_number);
 				current.cohorts.erase(current.cohorts.begin() + cohort->local_number);
 				foreach (iter, current.cohorts) {
-					(*iter)->local_number = static_cast<uint32_t>(std::distance(current.cohorts.begin(), iter));
+					(*iter)->local_number = UI32(std::distance(current.cohorts.begin(), iter));
 				}
 				gWindow->rebuildCohortLinks();
 			};
@@ -927,7 +933,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				auto nt = make_relation_rtag(tag, id);
 				for (auto& r : cohort->readings) {
 					for (auto it = r->tags_list.begin(); it != r->tags_list.end();) {
-						auto utag = single_tags[*it]->tag;
+						const auto& utag = grammar->single_tags[*it]->tag;
 						if (utag[0] == 'R' && utag[1] == ':' && utag.size() > 2 + tag->tag.size() && utag[2 + tag->tag.size()] == ':' && utag.compare(2, tag->tag.size(), tag->tag) == 0) {
 							r->tags.erase(*it);
 							r->tags_textual.erase(*it);
@@ -999,6 +1005,12 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					unif_tags = unif_tags_rs[reading.hash_plain];
 					unif_sets = unif_sets_rs[reading.hash_plain];
 				}
+				else {
+					unif_tags = &unif_tags_store[used_unif];
+					unif_sets = &unif_sets_store[used_unif];
+					clear(*unif_tags);
+					clear(*unif_sets);
+				}
 
 				auto insert_taglist_to_reading = [&](auto& iter, auto& taglist, auto& reading, auto& mappings) {
 					for (auto tag : taglist) {
@@ -1117,9 +1129,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 						auto ei = externals.find(rule.varname);
 						if (ei == externals.end()) {
-							Tag* ext = single_tags.find(rule.varname)->second;
+							Tag* ext = grammar->single_tags.find(rule.varname)->second;
 							UErrorCode err = U_ZERO_ERROR;
-							u_strToUTF8(&cbuffers[0][0], static_cast<int32_t>(CG3_BUFFER_SIZE - 1), nullptr, ext->tag.c_str(), static_cast<int32_t>(ext->tag.size()), &err);
+							u_strToUTF8(&cbuffers[0][0], SI32(CG3_BUFFER_SIZE - 1), nullptr, ext->tag.c_str(), SI32(ext->tag.size()), &err);
 
 							Process& es = externals[rule.varname];
 							try {
@@ -1230,9 +1242,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						std::vector<std::pair<uint32_t, uint32_t>> cohort_dep(cohorts.size());
 						cohort_dep.front().second = DEP_NO_PARENT;
 						cohort_dep.back().first = DEP_NO_PARENT;
-						cohort_dep.back().second = static_cast<uint32_t>(cohort_dep.size() - 1);
+						cohort_dep.back().second = UI32(cohort_dep.size() - 1);
 						for (size_t i = 1; i < cohort_dep.size() - 1; ++i) {
-							cohort_dep[i].second = static_cast<uint32_t>(i);
+							cohort_dep[i].second = UI32(i);
 						}
 
 						size_t i = 0;
@@ -1261,7 +1273,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								if (dep_self[0] == 'c' || dep_self[0] == 'd') {
 									cohort_dep[i - 1].first = DEP_NO_PARENT;
 									if (rel_trg == DEP_NO_PARENT) {
-										rel_trg = static_cast<uint32_t>(i - 1);
+										rel_trg = UI32(i - 1);
 									}
 								}
 								else if (u_sscanf(dep_self, "%i", &cohort_dep[i - 1].first) != 1) {
@@ -1278,14 +1290,14 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								continue;
 							}
 							if (tter->tag.size() == 3 && tter->tag[0] == 'R' && tter->tag[1] == ':' && tter->tag[2] == '*') {
-								rel_trg = static_cast<uint32_t>(i - 1);
+								rel_trg = UI32(i - 1);
 								continue;
 							}
 							readings->back().push_back(tter);
 						}
 
 						if (rel_trg == DEP_NO_PARENT) {
-							rel_trg = static_cast<uint32_t>(cohorts.size() - 1);
+							rel_trg = UI32(cohorts.size() - 1);
 						}
 
 						for (size_t i = 0; i < cohorts.size(); ++i) {
@@ -1307,12 +1319,12 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 											continue;
 										}
 										tags.reserve(tags.size() + nt.size() - 2);
-										tags[i] = single_tags[nt[2]];
+										tags[i] = grammar->single_tags[nt[2]];
 										for (size_t j = 3, k = 1; j < nt.size(); ++j) {
-											if (single_tags[nt[j]]->type & T_DEPENDENCY) {
+											if (grammar->single_tags[nt[j]]->type & T_DEPENDENCY) {
 												continue;
 											}
-											tags.insert(tags.begin() + i + k, single_tags[nt[j]]);
+											tags.insert(tags.begin() + i + k, grammar->single_tags[nt[j]]);
 											++k;
 										}
 									}
@@ -1408,7 +1420,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						cohort->type |= CT_REMOVED;
 						cohort->prev->removed.push_back(cohort);
 						cohort->detach();
-						for (auto cm : current.parent->cohort_map) {
+						for (auto& cm : current.parent->cohort_map) {
 							cm.second->dep_children.erase(cohort->dep_self);
 						}
 						current.parent->cohort_map.erase(cohort->global_number);
@@ -1416,7 +1428,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 						// Reindex and rebuild the window
 						foreach (iter, current.cohorts) {
-							(*iter)->local_number = static_cast<uint32_t>(std::distance(current.cohorts.begin(), iter));
+							(*iter)->local_number = UI32(std::distance(current.cohorts.begin(), iter));
 						}
 						gWindow->rebuildCohortLinks();
 						indexSingleWindow(current);
@@ -2084,7 +2096,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								}
 
 								foreach (iter, current.cohorts) {
-									(*iter)->local_number = static_cast<uint32_t>(std::distance(current.cohorts.begin(), iter));
+									(*iter)->local_number = UI32(std::distance(current.cohorts.begin(), iter));
 								}
 
 								for (auto iter : edges) {
@@ -2121,7 +2133,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 								}
 							}
 							foreach (iter, current.cohorts) {
-								(*iter)->local_number = static_cast<uint32_t>(std::distance(current.cohorts.begin(), iter));
+								(*iter)->local_number = UI32(std::distance(current.cohorts.begin(), iter));
 							}
 							gWindow->rebuildCohortLinks();
 
@@ -2453,12 +2465,13 @@ void GrammarApplicator::runGrammarOnWindow() {
 	SingleWindow* current = gWindow->current;
 	did_final_enclosure = false;
 
-	for (auto vit : current->variables_set) {
+	for (const auto& vit : current->variables_set) {
 		variables[vit.first] = vit.second;
 	}
 	for (auto vit : current->variables_rem) {
 		variables.erase(vit);
 	}
+	variables[mprefix_key] = mprefix_value;
 
 	if (has_dep) {
 		reflowDependencyWindow();
@@ -2586,18 +2599,18 @@ label_unpackEnclosures:
 				size_t ne = c->enclosed.size();
 				for (size_t j = nc - 1; j > i; --j) {
 					current->cohorts[j + ne] = current->cohorts[j];
-					current->cohorts[j + ne]->local_number = static_cast<uint32_t>(j + ne);
+					current->cohorts[j + ne]->local_number = UI32(j + ne);
 				}
 				for (size_t j = 0; j < ne; ++j) {
 					current->cohorts[i + j + 1] = c->enclosed[j];
-					current->cohorts[i + j + 1]->local_number = static_cast<uint32_t>(i + j + 1);
+					current->cohorts[i + j + 1]->local_number = UI32(i + j + 1);
 					current->cohorts[i + j + 1]->parent = current;
 					current->cohorts[i + j + 1]->type &= ~CT_ENCLOSED;
 				}
 				par_left_tag = c->enclosed[0]->is_pleft;
 				par_right_tag = c->enclosed[ne - 1]->is_pright;
-				par_left_pos = static_cast<uint32_t>(i + 1);
-				par_right_pos = static_cast<uint32_t>(i + ne);
+				par_left_pos = UI32(i + 1);
+				par_right_pos = UI32(i + ne);
 				c->enclosed.clear();
 				if (rv & RV_TRACERULE) {
 					goto label_unpackEnclosures;
