@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -32,12 +32,6 @@ namespace CG3 {
 MatxinApplicator::MatxinApplicator(std::ostream& ux_err)
   : GrammarApplicator(ux_err)
 {
-	nullFlush = false;
-	wordform_case = false;
-	unique_tags = false;
-	print_word_forms = true;
-	print_only_first = false;
-	runningWithNullFlush = false;
 }
 
 bool MatxinApplicator::getNullFlush() {
@@ -581,7 +575,7 @@ void MatxinApplicator::processReading(Reading* cReading, const UChar* reading_st
 				}
 				// Add tags
 				TagList mappings;
-				TagVector::iterator iter = riter.base();
+				auto iter = riter.base();
 				for (--iter; iter != taglist.end(); ++iter) {
 					if ((*iter)->type & T_MAPPING || (*iter)->tag[0] == grammar->mapping_prefix) {
 						mappings.push_back(*iter);
@@ -633,7 +627,7 @@ void MatxinApplicator::printReading(Reading* reading, Node& node, std::ostream&
 	}
 
 	// Lop off the initial and final '"' characters
-	UnicodeString bf(single_tags[reading->baseform]->tag.c_str() + 1, static_cast<int32_t>(single_tags[reading->baseform]->tag.size() - 2));
+	UnicodeString bf(grammar->single_tags[reading->baseform]->tag.c_str() + 1, SI32(grammar->single_tags[reading->baseform]->tag.size() - 2));
 
 	node.lemma = bf.getTerminatedBuffer();
 
@@ -645,7 +639,7 @@ void MatxinApplicator::printReading(Reading* reading, Node& node, std::ostream&
 	bool multi = false;
 	bool first = true;
 	for (auto tter : reading->tags_list) {
-		const Tag* tag = single_tags[tter];
+		const Tag* tag = grammar->single_tags[tter];
 		if (tag->tag[0] == '+') {
 			multi = true;
 		}
@@ -675,7 +669,7 @@ void MatxinApplicator::printReading(Reading* reading, Node& node, std::ostream&
 		if (tter == endtag || tter == begintag) {
 			continue;
 		}
-		const Tag* tag = single_tags[tter];
+		const Tag* tag = grammar->single_tags[tter];
 		if (!(tag->type & T_BASEFORM) && !(tag->type & T_WORDFORM)) {
 			if (tag->tag[0] == '+') {
 				u_fprintf(output, "%S", tag->tag.c_str());
@@ -727,7 +721,7 @@ void MatxinApplicator::printSingleWindow(SingleWindow* window, std::ostream& out
 
 		// Lop off the initial and final '"' characters
 		// ToDo: A copy does not need to be made here - use pointer offsets
-		UnicodeString wf(cohort->wordform->tag.c_str() + 2, static_cast<int32_t>(cohort->wordform->tag.size() - 4));
+		UnicodeString wf(cohort->wordform->tag.c_str() + 2, SI32(cohort->wordform->tag.size() - 4));
 		UString wf_escaped;
 		for (int i = 0; i < wf.length(); ++i) {
 			if (wf[i] == '&') {
@@ -758,13 +752,13 @@ void MatxinApplicator::printSingleWindow(SingleWindow* window, std::ostream& out
 				if (tter == cohort->wordform->hash) {
 					continue;
 				}
-				const Tag *tag = single_tags[tter];
+				const Tag *tag = grammar->single_tags[tter];
 				u_fprintf(output, "<%S>", tag->tag.c_str());
 			}
 		}
 //*/
 
-		//Tag::printTagRaw(output, single_tags[cohort->wordform]);
+		//Tag::printTagRaw(output, grammar->single_tags[cohort->wordform]);
 		Reading* reading = cohort->readings[0];
 
 		printReading(reading, n, output);
