@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -31,8 +31,6 @@ namespace CG3 {
 
 NicelineApplicator::NicelineApplicator(std::ostream& ux_err)
   : GrammarApplicator(ux_err)
-  , did_warn_statictags(false)
-  , did_warn_subreadings(false)
 {
 }
 
@@ -89,39 +87,8 @@ void NicelineApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 
 	while (!input.eof()) {
 		++lines;
-		size_t offset = 0, packoff = 0;
-		// Read as much of the next line as will fit in the current buffer
-		while (u_fgets(&line[offset], static_cast<int32_t>(line.size() - offset - 1), input)) {
-			// Copy the segment just read to cleaned
-			for (size_t i = offset; i < line.size(); ++i) {
-				// Only copy one space character, regardless of how many are in input
-				if (ISSPACE(line[i]) && !ISNL(line[i])) {
-					cleaned[packoff++] = (line[i] == '\t' ? '\t' : ' ');
-					while (ISSPACE(line[i]) && !ISNL(line[i])) {
-						if (line[i] == '\t') {
-							cleaned[packoff - 1] = line[i];
-						}
-						++i;
-					}
-				}
-				// Break if there is a newline
-				if (ISNL(line[i])) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					goto gotaline; // Oh how I wish C++ had break 2;
-				}
-				if (line[i] == 0) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					break;
-				}
-				cleaned[packoff++] = line[i];
-			}
-			// If we reached this, buffer wasn't big enough. Double the size of the buffer and try again.
-			offset = line.size() - 2;
-			line.resize(line.size() * 2, 0);
-			cleaned.resize(line.size() + 2, 0);
-		}
+		auto packoff = get_line_clean(line, cleaned, input, true);
 
-	gotaline:
 		// Trim trailing whitespace
 		while (cleaned[0] && ISSPACE(cleaned[packoff - 1])) {
 			cleaned[packoff - 1] = 0;
@@ -359,7 +326,7 @@ void NicelineApplicator::printReading(const Reading* reading, std::ostream& outp
 	}
 	u_fputc('\t', output);
 	if (reading->baseform) {
-		u_fprintf(output, "[%.*S]", single_tags.find(reading->baseform)->second->tag.size() - 2, single_tags.find(reading->baseform)->second->tag.c_str() + 1);
+		u_fprintf(output, "[%.*S]", grammar->single_tags.find(reading->baseform)->second->tag.size() - 2, grammar->single_tags.find(reading->baseform)->second->tag.c_str() + 1);
 	}
 
 	uint32SortedVector unique;
@@ -376,7 +343,7 @@ void NicelineApplicator::printReading(const Reading* reading, std::ostream& outp
 			}
 			unique.insert(tter);
 		}
-		const Tag* tag = single_tags[tter];
+		const Tag* tag = grammar->single_tags[tter];
 		if (tag->type & T_DEPENDENCY && has_dep && !dep_original) {
 			continue;
 		}
@@ -432,7 +399,7 @@ void NicelineApplicator::printReading(const Reading* reading, std::ostream& outp
 	if (reading->parent->type & CT_RELATED) {
 		u_fprintf(output, " ID:%u", reading->parent->global_number);
 		if (!reading->parent->relations.empty()) {
-			for (auto miter : reading->parent->relations) {
+			for (const auto& miter : reading->parent->relations) {
 				for (auto siter : miter.second) {
 					u_fprintf(output, " R:%S:%u", grammar->single_tags.find(miter.first)->second->tag.c_str(), siter);
 				}
@@ -500,7 +467,7 @@ void NicelineApplicator::printSingleWindow(SingleWindow* window, std::ostream& o
 		}
 	}
 
-	uint32_t cs = static_cast<uint32_t>(window->cohorts.size());
+	uint32_t cs = UI32(window->cohorts.size());
 	for (uint32_t c = 0; c < cs; c++) {
 		Cohort* cohort = window->cohorts[c];
 		printCohort(cohort, output);
