@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -27,25 +27,13 @@ namespace CG3 {
 
 std::ostream* Set::dump_hashes_out = nullptr;
 
-Set::Set()
-  : type(0)
-  , line(0)
-  , hash(0)
-  , number(0)
-  , num_fail(0)
-  , num_match(0)
-  , total_time(0)
-{
-	// Nothing in the actual body...
-}
-
 bool Set::empty() const {
 	return (ff_tags.empty() && trie.empty() && trie_special.empty() && sets.empty());
 }
 
 void Set::setName(uint32_t to) {
 	if (!to) {
-		to = static_cast<uint32_t>(rand());
+		to = UI32(rand());
 	}
 	size_t n = sprintf(&cbuffers[0][0], "_G_%u_%u_", line, to);
 	name.reserve(n);
