@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -28,24 +28,6 @@ namespace CG3 {
 
 std::ostream* Tag::dump_hashes_out = nullptr;
 
-Tag::Tag()
-  : comparison_op(OP_NOP)
-  , comparison_val(0)
-  , type(0)
-  , comparison_hash(0)
-  , dep_self(0)
-  , dep_parent(0)
-  , hash(0)
-  , plain_hash(0)
-  , number(0)
-  , seed(0)
-  , regexp(nullptr)
-{
-	#ifdef CG_TRACE_OBJECTS
-	std::cerr << "OBJECT: " << __PRETTY_FUNCTION__ << std::endl;
-	#endif
-}
-
 Tag::Tag(const Tag& o)
   : comparison_op(o.comparison_op)
   , comparison_val(o.comparison_val)
@@ -91,16 +73,14 @@ Tag::~Tag() {
 
 void Tag::parseTagRaw(const UChar* to, Grammar* grammar) {
 	type = 0;
-	size_t length = u_strlen(to);
+	const size_t length = u_strlen(to);
 	assert(length && "parseTagRaw() will not work with empty strings.");
 
-	const UChar* tmp = to;
-
-	if (tmp[0] && (tmp[0] == '"' || tmp[0] == '<')) {
-		if ((tmp[0] == '"' && tmp[length - 1] == '"') || (tmp[0] == '<' && tmp[length - 1] == '>')) {
+	if (to[0] && (to[0] == '"' || to[0] == '<')) {
+		if ((to[0] == '"' && to[length - 1] == '"') || (to[0] == '<' && to[length - 1] == '>')) {
 			type |= T_TEXTUAL;
-			if (tmp[0] == '"' && tmp[length - 1] == '"') {
-				if (tmp[1] == '<' && tmp[length - 2] == '>' && length > 4) {
+			if (to[0] == '"' && to[length - 1] == '"') {
+				if (to[1] == '<' && to[length - 2] == '>' && length > 4) {
 					type |= T_WORDFORM;
 				}
 				else {
@@ -110,11 +90,11 @@ void Tag::parseTagRaw(const UChar* to, Grammar* grammar) {
 		}
 	}
 
-	tag.assign(tmp, length);
+	tag.assign(to, length);
 
 	for (auto iter : grammar->regex_tags) {
 		UErrorCode status = U_ZERO_ERROR;
-		uregex_setText(iter, tag.c_str(), static_cast<int32_t>(tag.size()), &status);
+		uregex_setText(iter, tag.c_str(), SI32(tag.size()), &status);
 		if (status == U_ZERO_ERROR) {
 			if (uregex_find(iter, -1, &status)) {
 				type |= T_TEXTUAL;
