@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -186,7 +186,7 @@ int main(int argc, char* argv[]) {
 
 		CG3::UString buffer(BUF_SIZE, 0);
 		int32_t nr = 0;
-		u_strFromUTF8(&buffer[0], BUF_SIZE, &nr, buf8.c_str(), static_cast<int32_t>(sz), &status);
+		u_strFromUTF8(&buffer[0], BUF_SIZE, &nr, buf8.c_str(), SI32(sz), &status);
 		if (U_FAILURE(status)) {
 			throw std::runtime_error("UTF-8 to UTF-16 conversion failed");
 		}
@@ -195,7 +195,7 @@ int main(int argc, char* argv[]) {
 
 		for (;;) {
 			rx = uregex_openC("^\"<[^>]+>\".*?^\\s+\"[^\"]+\"", UREGEX_DOTALL | UREGEX_MULTILINE, 0, &status);
-			uregex_setText(rx, buffer.c_str(), static_cast<int32_t>(buffer.size()), &status);
+			uregex_setText(rx, buffer.c_str(), SI32(buffer.size()), &status);
 			if (uregex_find(rx, -1, &status)) {
 				fmt = CG3::FMT_CG;
 				break;
@@ -203,7 +203,7 @@ int main(int argc, char* argv[]) {
 			uregex_close(rx);
 
 			rx = uregex_openC("^\\S+ *\t *\\[\\S+\\]", UREGEX_DOTALL | UREGEX_MULTILINE, 0, &status);
-			uregex_setText(rx, buffer.c_str(), static_cast<int32_t>(buffer.size()), &status);
+			uregex_setText(rx, buffer.c_str(), SI32(buffer.size()), &status);
 			if (uregex_find(rx, -1, &status)) {
 				fmt = CG3::FMT_NICELINE;
 				break;
@@ -211,7 +211,7 @@ int main(int argc, char* argv[]) {
 			uregex_close(rx);
 
 			rx = uregex_openC("^\\S+ *\t *\"\\S+\"", UREGEX_DOTALL | UREGEX_MULTILINE, 0, &status);
-			uregex_setText(rx, buffer.c_str(), static_cast<int32_t>(buffer.size()), &status);
+			uregex_setText(rx, buffer.c_str(), SI32(buffer.size()), &status);
 			if (uregex_find(rx, -1, &status)) {
 				fmt = CG3::FMT_NICELINE;
 				break;
@@ -219,7 +219,7 @@ int main(int argc, char* argv[]) {
 			uregex_close(rx);
 
 			rx = uregex_openC("\\^[^/]+(/[^<]+(<[^>]+>)+)+\\$", UREGEX_DOTALL | UREGEX_MULTILINE, 0, &status);
-			uregex_setText(rx, buffer.c_str(), static_cast<int32_t>(buffer.size()), &status);
+			uregex_setText(rx, buffer.c_str(), SI32(buffer.size()), &status);
 			if (uregex_find(rx, -1, &status)) {
 				fmt = CG3::FMT_APERTIUM;
 				break;
@@ -227,7 +227,7 @@ int main(int argc, char* argv[]) {
 			uregex_close(rx);
 
 			rx = uregex_openC("^\\S+\t\\S+(\\+\\S+)+$", UREGEX_DOTALL | UREGEX_MULTILINE, 0, &status);
-			uregex_setText(rx, buffer.c_str(), static_cast<int32_t>(buffer.size()), &status);
+			uregex_setText(rx, buffer.c_str(), SI32(buffer.size()), &status);
 			if (uregex_find(rx, -1, &status)) {
 				fmt = CG3::FMT_FST;
 				break;
@@ -248,27 +248,27 @@ int main(int argc, char* argv[]) {
 		grammar.sub_readings_ltr = true;
 	}
 	if (options[MAPPING_PREFIX].doesOccur) {
-		auto sn = static_cast<int32_t>(strlen(options[MAPPING_PREFIX].value));
+		auto sn = SI32(strlen(options[MAPPING_PREFIX].value));
 		CG3::UString buf(sn * 3, 0);
 		UConverter* conv = ucnv_open(codepage_default, &status);
-		ucnv_toUChars(conv, &buf[0], static_cast<int32_t>(buf.size()), options[MAPPING_PREFIX].value, sn, &status);
+		ucnv_toUChars(conv, &buf[0], SI32(buf.size()), options[MAPPING_PREFIX].value, sn, &status);
 		ucnv_close(conv);
 		grammar.mapping_prefix = buf[0];
 	}
 	if (options[SUB_DELIMITER].doesOccur) {
-		auto sn = static_cast<int32_t>(strlen(options[SUB_DELIMITER].value));
+		auto sn = SI32(strlen(options[SUB_DELIMITER].value));
 		applicator.sub_delims.resize(sn * 2);
 		UConverter* conv = ucnv_open(codepage_default, &status);
-		sn = ucnv_toUChars(conv, &applicator.sub_delims[0], static_cast<int32_t>(applicator.sub_delims.size()), options[SUB_DELIMITER].value, sn, &status);
+		sn = ucnv_toUChars(conv, &applicator.sub_delims[0], SI32(applicator.sub_delims.size()), options[SUB_DELIMITER].value, sn, &status);
 		applicator.sub_delims.resize(sn);
 		applicator.sub_delims += '+';
 		ucnv_close(conv);
 	}
 	if (options[FST_WTAG].doesOccur) {
-		auto sn = static_cast<int32_t>(strlen(options[FST_WTAG].value));
+		auto sn = SI32(strlen(options[FST_WTAG].value));
 		applicator.wtag.resize(sn * 2);
 		UConverter* conv = ucnv_open(codepage_default, &status);
-		sn = ucnv_toUChars(conv, &applicator.wtag[0], static_cast<int32_t>(applicator.wtag.size()), options[FST_WTAG].value, sn, &status);
+		sn = ucnv_toUChars(conv, &applicator.wtag[0], SI32(applicator.wtag.size()), options[FST_WTAG].value, sn, &status);
 		applicator.wtag.resize(sn);
 		ucnv_close(conv);
 	}
@@ -280,6 +280,7 @@ int main(int argc, char* argv[]) {
 
 	if (options[OUT_APERTIUM].doesOccur) {
 		applicator.setOutputFormat(CG3::FMT_APERTIUM);
+		applicator.unicode_tags = true;
 	}
 	else if (options[OUT_NICELINE].doesOccur) {
 		applicator.setOutputFormat(CG3::FMT_NICELINE);
@@ -288,6 +289,9 @@ int main(int argc, char* argv[]) {
 		applicator.setOutputFormat(CG3::FMT_PLAIN);
 	}
 
+	if (options[UNICODE_TAGS].doesOccur) {
+		applicator.unicode_tags = true;
+	}
 	if (options[PIPE_DELETED].doesOccur) {
 		applicator.pipe_deleted = true;
 	}
@@ -295,6 +299,11 @@ int main(int argc, char* argv[]) {
 		applicator.add_spacing = false;
 	}
 
+
+	if (options[PARSE_DEP].doesOccur) {
+		applicator.parse_dep = true;
+		applicator.has_dep = true;
+	}
 	applicator.is_conv = true;
 	applicator.trace = true;
 	applicator.verbosity_level = 0;
