@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -36,7 +36,7 @@ int main(int argc, char* argv[]) {
 	clock_t main_timer = clock();
 
 	UErrorCode status = U_ZERO_ERROR;
-	srand(static_cast<uint32_t>(time(0)));
+	srand(UI32(time(0)));
 
 	U_MAIN_INIT_ARGS(argc, argv);
 	argc = u_parseArgs(argc, argv, NUM_OPTIONS, options.data());
@@ -266,7 +266,7 @@ int main(int argc, char* argv[]) {
 		UConverter* conv = ucnv_open(codepage_cli, &status);
 		size_t sn = strlen(options[MAPPING_PREFIX].value);
 		CG3::UString buf(sn * 3, 0);
-		ucnv_toUChars(conv, &buf[0], static_cast<int32_t>(buf.size()), options[MAPPING_PREFIX].value, static_cast<int32_t>(sn), &status);
+		ucnv_toUChars(conv, &buf[0], SI32(buf.size()), options[MAPPING_PREFIX].value, SI32(sn), &status);
 		if (grammar.is_binary && grammar.mapping_prefix != buf[0]) {
 			std::cerr << "Error: Mapping prefix must match the one used for compiling the binary grammar!" << std::endl;
 			CG3Quit(1);
@@ -355,7 +355,7 @@ int main(int argc, char* argv[]) {
 			grammar.rule_by_number.erase(grammar.rule_by_number.begin() + (*br)->number);
 		}
 		for (auto br : bad) {
-			br->number = static_cast<uint32_t>(grammar.rule_by_number.size());
+			br->number = UI32(grammar.rule_by_number.size());
 			grammar.rule_by_number.push_back(br);
 		}
 		std::cerr << "Optimizer moved " << bad.size() << " rules." << std::endl;
@@ -491,7 +491,7 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter* conv) {
 			UErrorCode status = U_ZERO_ERROR;
 			size_t sn = strlen(options[RULE].value);
 			CG3::UString buf(sn * 3, 0);
-			ucnv_toUChars(conv, &buf[0], static_cast<int32_t>(sn * 3), options[RULE].value, static_cast<int32_t>(sn), &status);
+			ucnv_toUChars(conv, &buf[0], SI32(sn * 3), options[RULE].value, SI32(sn), &status);
 
 			for (auto rule : applicator.grammar->rule_by_number) {
 				if (rule->name == buf) {
@@ -532,7 +532,7 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter* conv) {
 			UErrorCode status = U_ZERO_ERROR;
 			size_t sn = strlen(options[TEXT_DELIMIT].value);
 			CG3::UString buf(sn * 3, 0);
-			auto len = ucnv_toUChars(conv, &buf[0], static_cast<int32_t>(sn * 3), options[TEXT_DELIMIT].value, static_cast<int32_t>(sn), &status);
+			auto len = ucnv_toUChars(conv, &buf[0], SI32(sn * 3), options[TEXT_DELIMIT].value, SI32(sn), &status);
 			rx.assign(buf.begin(), buf.begin() + len);
 		}
 		applicator.setTextDelimiter(rx);
