@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -50,6 +50,8 @@ enum OPTIONS {
 	SUB_RTL,
 	SUB_LTR,
 	ORDERED,
+	PARSE_DEP,
+	UNICODE_TAGS,
 	PIPE_DELETED,
 	NO_BREAK,
 	NUM_OPTIONS,
@@ -79,6 +81,8 @@ std::array<UOption, NUM_OPTIONS> options{
 	UOPTION_DEF_D("rtl",          'r', UOPT_NO_ARG,       "sets sub-reading direction to RTL (default)"),
 	UOPTION_DEF_D("ltr",          'l', UOPT_NO_ARG,       "sets sub-reading direction to LTR"),
 	UOPTION_DEF_D("ordered",      'o', UOPT_NO_ARG,       "tag order matters mode"),
+	UOPTION_DEF_D("parse-dep",    'D', UOPT_NO_ARG,       "parse dependency (defaults to treating as normal tags)"),
+	UOPTION_DEF_D("unicode-tags",   0, UOPT_NO_ARG,       "outputs Unicode code points for things like ->"),
 	UOPTION_DEF_D("deleted",        0, UOPT_NO_ARG,       "read deleted readings as such, instead of as text"),
 	UOPTION_DEF_D("no-break",     'B', UOPT_NO_ARG,       "inhibits any extra whitespace in output"),
 };
