@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -74,6 +74,7 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 		}
 		if (tmp[0] == 'V' && tmp[1] == 'A' && tmp[2] == 'R' && tmp[3] == ':') {
 			tag->type |= T_VARIABLE;
+			tag->variable_hash = 0;
 			tmp += 4;
 			length -= 4;
 		}
@@ -174,7 +175,7 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 
 		for (auto iter : state.get_grammar()->regex_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(iter, tag->tag.c_str(), static_cast<int32_t>(tag->tag.size()), &status);
+			uregex_setText(iter, tag->tag.c_str(), SI32(tag->tag.size()), &status);
 			if (status != U_ZERO_ERROR) {
 				state.error("%s: Error: uregex_setText(parseTag) returned %s on line %u near `%S` - cannot continue!\n", u_errorName(status), p);
 			}
@@ -189,7 +190,15 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 			}
 		}
 
-		tag->comparison_hash = hash_value(tag->tag);
+		if ((tag->type & T_VARIABLE) && tag->tag.find('=') != UString::npos) {
+			size_t pos = tag->tag.find('=');
+			tag->comparison_op = OP_EQUALS;
+			tag->variable_hash = parseTag(&tag->tag[pos + 1], p, state)->hash;
+			tag->comparison_hash = hash_value(tag->tag.substr(0, pos));
+		}
+		else {
+			tag->comparison_hash = hash_value(tag->tag);
+		}
 
 		if (tag->tag[0] == '<' && tag->tag[length - 1] == '>') {
 			tag->parseNumeric();
@@ -254,10 +263,10 @@ Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 				}
 
 				if (tag->type & T_CASE_INSENSITIVE) {
-					tag->regexp = uregex_open(rt.c_str(), static_cast<int32_t>(rt.size()), UREGEX_CASE_INSENSITIVE, &pe, &status);
+					tag->regexp = uregex_open(rt.c_str(), SI32(rt.size()), UREGEX_CASE_INSENSITIVE, &pe, &status);
 				}
 				else {
-					tag->regexp = uregex_open(rt.c_str(), static_cast<int32_t>(rt.size()), 0, &pe, &status);
+					tag->regexp = uregex_open(rt.c_str(), SI32(rt.size()), 0, &pe, &status);
 				}
 				if (status != U_ZERO_ERROR) {
 					state.error("%s: Error: uregex_open returned %s trying to parse tag %S on line %u near `%S` - cannot continue!\n", u_errorName(status), tag->tag.c_str(), p);
