@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -85,6 +85,8 @@ inline bool ux_stripBOM(std::istream& stream) {
 	return true;
 }
 
+size_t get_line_clean(CG3::UString& line, CG3::UString& cleaned, std::istream& input, bool keep_tabs = false);
+
 // ICU std::istream input wrappers
 UChar* u_fgets(UChar* s, int32_t n, std::istream& input);
 
@@ -118,9 +120,9 @@ inline int ux_isSetOp(const UChar* it) {
 			return S_FAILFAST;
 		case '\\':
 			return S_SET_DIFF;
-		case 8745:
+		case u'\u2229':
 			return S_SET_ISECT_U;
-		case 8710:
+		case u'\u2206':
 			return S_SET_SYMDIFF_U;
 		default:
 			break;
@@ -178,7 +180,7 @@ inline bool ux_simplecasecmp(const UChar* a, const UString& b) {
 
 inline bool ux_strCaseCompare(const UString& a, const UString& b) {
 	UErrorCode status = U_ZERO_ERROR;
-	auto rv = u_strCaseCompare(a.c_str(), static_cast<int32_t>(a.size()), b.c_str(), static_cast<int32_t>(b.size()), U_FOLD_CASE_DEFAULT, &status);
+	auto rv = u_strCaseCompare(a.c_str(), SI32(a.size()), b.c_str(), SI32(b.size()), U_FOLD_CASE_DEFAULT, &status);
 	if (status != U_ZERO_ERROR) {
 		throw new std::runtime_error(u_errorName(status));
 	}
@@ -235,6 +237,7 @@ inline UChar* ux_bufcpy(UChar* dst, const UChar* src, size_t n) {
 }
 
 std::string ux_dirname(const char* in);
+void findAndReplace(UnicodeString& str, CG3::UStringView from, CG3::UStringView to);
 }
 
 #endif
